function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

import omit from 'ramda/src/omit';
import Promise from 'bluebird';

import { readData, writeData } from './fixtures';

import Logger from '../../util/log';
var log = Logger`simple-file-storage`;

/**
 * ### *Basic* file storage realisation
 *
 * Limitations:
 * * `filepath` must exists
 * * `filepath` must be valid JSON (just `{}` for empty store)
 * * No auto close descriptor
 *
 * @export
 * @class SimpleFileStorage
 * @implements {AsyncStorage}
 */
export class SimpleFileStorage {

  /**
   * Usage
   *
   *     const storage = new SimpleFileStorage('./filestore.json')
   *     const telegram = MTProto({ app: { storage } })
   *
   * @param {string} filepath Path to your file
   * @param {object} [data] Import stored data (if you have so)
   *
   */
  constructor(filepath, data) {
    this.data = {};
    this.init = false;

    this.filepath = filepath;
    if (data != null) this.data = data;
    setInterval(() => this.save(), 500);
  }

  save() {
    var str = JSON.stringify(this.data);
    return writeData(this.filepath, str);
  }

  get(key) {
    var _this = this;

    return _asyncToGenerator(function* () {
      if (!_this.init) {
        var pureData = yield readData(_this.filepath);
        var parsed = JSON.parse(pureData);
        _this.data = parsed;
        log('get', 'parsed')(parsed);
        _this.init = Date.now();
      }
      var data = _this.data[key];
      log('get', `key ${key}`)(data);
      return Promise.resolve(data);
    })();
  }

  set(key, val) {
    this.data[key] = val;
    return Promise.resolve();
  }

  remove(...keys) {
    this.data = omit(keys, this.data);
    return Promise.resolve();
  }

  clear() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      _this2.data = {};
      yield _this2.save();
    })();
  }
}

export default SimpleFileStorage;
//# sourceMappingURL=simple-file-storage.js.map